@@ -12,7 +12,8 @@ import glob
 import token
 import tokenize
 import shutil
-import json
+
+from seaborn.external import six
 
 import matplotlib
 matplotlib.use('Agg')
@@ -20,6 +21,12 @@ import matplotlib.pyplot as plt
 
 from matplotlib import image
 
+if six.PY3:
+    # Python 3 has no execfile
+    def execfile(filename, globals=None, locals=None):
+        with open(filename, "rb") as fp:
+            six.exec_(compile(fp.read(), filename, 'exec'), globals, locals)
+
 
 RST_TEMPLATE = """
 .. _{sphinx_tag}:
@@ -117,7 +124,7 @@ Example gallery
 
 
 def create_thumbnail(infile, thumbfile,
-                     width=300, height=300,
+                     width=275, height=275,
                      cx=0.5, cy=0.5, border=4):
     baseout, extout = op.splitext(thumbfile)
 
@@ -235,7 +242,8 @@ class ExampleGenerator(object):
 
         docstring = ''
         first_par = ''
-        tokens = tokenize.generate_tokens(lines.__iter__().next)
+        line_iter = lines.__iter__()
+        tokens = tokenize.generate_tokens(lambda: next(line_iter))
         for tok_type, tok_content, _, (erow, _), _ in tokens:
             tok_type = token.tok_name[tok_type]
             if tok_type in ('NEWLINE', 'COMMENT', 'NL', 'INDENT', 'DEDENT'):
