@@ -9,22 +9,27 @@ import matplotlib.pyplot as plt
 sns.set(style="white", context="talk")
 rs = np.random.RandomState(7)
 
-x = np.array(list("ABCDEFGHI"))
 
+# Set up the matplotlib figure
 f, (ax1, ax2, ax3) = plt.subplots(3, 1, figsize=(8, 6), sharex=True)
 
+# Generate some sequential data
+x = np.array(list("ABCDEFGHI"))
 y1 = np.arange(1, 10)
-sns.barplot(x, y1, ci=None, palette="BuGn_d", hline=.1, ax=ax1)
+sns.barplot(x, y1, palette="BuGn_d", ax=ax1)
 ax1.set_ylabel("Sequential")
 
+# Center the data to make it diverging
 y2 = y1 - 5
-sns.barplot(x, y2, ci=None, palette="coolwarm", hline=0, ax=ax2)
+sns.barplot(x, y2, palette="RdBu_r", ax=ax2)
 ax2.set_ylabel("Diverging")
 
+# Randomly reorder the data to make it qualitative
 y3 = rs.choice(y1, 9, replace=False)
-sns.barplot(x, y3, ci=None, palette="Paired", hline=.1, ax=ax3)
+sns.barplot(x, y3, palette="Set3", ax=ax3)
 ax3.set_ylabel("Qualitative")
 
+# Finalize the plot
 sns.despine(bottom=True)
 plt.setp(f.axes, yticks=[])
 plt.tight_layout(h_pad=3)
