@@ -3,6 +3,7 @@ Annotated heatmaps
 ==================
 
 """
+import matplotlib.pyplot as plt
 import seaborn as sns
 sns.set()
 
@@ -11,4 +12,5 @@ flights_long = sns.load_dataset("flights")
 flights = flights_long.pivot("month", "year", "passengers")
 
 # Draw a heatmap with the numeric values in each cell
-sns.heatmap(flights, annot=True, fmt="d", linewidths=.5)
+f, ax = plt.subplots(figsize=(9, 6))
+sns.heatmap(flights, annot=True, fmt="d", linewidths=.5, ax=ax)
