@@ -11,7 +11,7 @@ sns.set(style="whitegrid")
 f, ax = plt.subplots(figsize=(6, 15))
 
 # Load the example car crash dataset
-crashes = sns.load_dataset("car_crashes").sort("total", ascending=False)
+crashes = sns.load_dataset("car_crashes").sort_values("total", ascending=False)
 
 # Plot the total crashes
 sns.set_color_codes("pastel")
