@@ -4,7 +4,7 @@ Plotting a diagonal correlation matrix
 
 _thumb: .3, .6
 """
-from string import letters
+from string import ascii_letters
 import numpy as np
 import pandas as pd
 import seaborn as sns
@@ -15,7 +15,7 @@ sns.set(style="white")
 # Generate a large random dataset
 rs = np.random.RandomState(33)
 d = pd.DataFrame(data=rs.normal(size=(100, 26)),
-                 columns=list(letters[:26]))
+                 columns=list(ascii_letters[26:]))
 
 # Compute the correlation matrix
 corr = d.corr()
@@ -31,6 +31,5 @@ f, ax = plt.subplots(figsize=(11, 9))
 cmap = sns.diverging_palette(220, 10, as_cmap=True)
 
 # Draw the heatmap with the mask and correct aspect ratio
-sns.heatmap(corr, mask=mask, cmap=cmap, vmax=.3,
-            square=True, xticklabels=5, yticklabels=5,
-            linewidths=.5, cbar_kws={"shrink": .5}, ax=ax)
+sns.heatmap(corr, mask=mask, cmap=cmap, vmax=.3, center=0,
+            square=True, linewidths=.5, cbar_kws={"shrink": .5})
