@@ -10,20 +10,23 @@ from distutils.version import LooseVersion
 import nose.tools as nt
 import numpy.testing as npt
 from numpy.testing.decorators import skipif
-import pandas.util.testing as tm
+try:
+    import pandas.testing as tm
+except ImportError:
+    import pandas.util.testing as tm
 
 from . import PlotTestCase
 from .. import axisgrid as ag
 from .. import rcmod
 from ..palettes import color_palette
-from ..distributions import kdeplot
+from ..distributions import kdeplot, _freedman_diaconis_bins
 from ..categorical import pointplot
-from ..linearmodels import pairplot
 from ..utils import categorical_order
 
 rs = np.random.RandomState(0)
 
 old_matplotlib = LooseVersion(mpl.__version__) < "1.4"
+pandas_has_categoricals = LooseVersion(pd.__version__) >= "0.15"
 
 
 class TestFacetGrid(PlotTestCase):
@@ -266,9 +269,10 @@ class TestFacetGrid(PlotTestCase):
 
     def test_subplot_kws(self):
 
-        g = ag.FacetGrid(self.df, subplot_kws=dict(axisbg="blue"))
+        g = ag.FacetGrid(self.df, despine=False,
+                         subplot_kws=dict(projection="polar"))
         for ax in g.axes.flat:
-            nt.assert_equal(ax.get_axis_bgcolor(), "blue")
+            nt.assert_true("PolarAxesSubplot" in str(type(ax)))
 
     @skipif(old_matplotlib)
     def test_gridspec_kws(self):
@@ -580,6 +584,149 @@ class TestFacetGrid(PlotTestCase):
         g = ag.FacetGrid(df, dropna=True, row="hasna")
         nt.assert_equal(g._not_na.sum(), 50)
 
+    def test_unicode_column_label_with_rows(self):
+
+        # use a smaller copy of the default testing data frame:
+        df = self.df.copy()
+        df = df[["a", "b", "x"]]
+
+        # rename column 'a' (which will be used for the columns in the grid)
+        # by using a Unicode string:
+        unicode_column_label = u"\u01ff\u02ff\u03ff"
+        df = df.rename(columns={"a": unicode_column_label})
+
+        # ensure that the data frame columns have the expected names:
+        nt.assert_equal(list(df.columns), [unicode_column_label, "b", "x"])
+
+        # plot the grid -- if successful, no UnicodeEncodingError should
+        # occur:
+        g = ag.FacetGrid(df, col=unicode_column_label, row="b")
+        g = g.map(plt.plot, "x")
+
+    def test_unicode_column_label_no_rows(self):
+
+        # use a smaller copy of the default testing data frame:
+        df = self.df.copy()
+        df = df[["a", "x"]]
+
+        # rename column 'a' (which will be used for the columns in the grid)
+        # by using a Unicode string:
+        unicode_column_label = u"\u01ff\u02ff\u03ff"
+        df = df.rename(columns={"a": unicode_column_label})
+
+        # ensure that the data frame columns have the expected names:
+        nt.assert_equal(list(df.columns), [unicode_column_label, "x"])
+
+        # plot the grid -- if successful, no UnicodeEncodingError should
+        # occur:
+        g = ag.FacetGrid(df, col=unicode_column_label)
+        g = g.map(plt.plot, "x")
+
+    def test_unicode_row_label_with_columns(self):
+
+        # use a smaller copy of the default testing data frame:
+        df = self.df.copy()
+        df = df[["a", "b", "x"]]
+
+        # rename column 'b' (which will be used for the rows in the grid)
+        # by using a Unicode string:
+        unicode_row_label = u"\u01ff\u02ff\u03ff"
+        df = df.rename(columns={"b": unicode_row_label})
+
+        # ensure that the data frame columns have the expected names:
+        nt.assert_equal(list(df.columns), ["a", unicode_row_label, "x"])
+
+        # plot the grid -- if successful, no UnicodeEncodingError should
+        # occur:
+        g = ag.FacetGrid(df, col="a", row=unicode_row_label)
+        g = g.map(plt.plot, "x")
+
+    def test_unicode_row_label_no_columns(self):
+
+        # use a smaller copy of the default testing data frame:
+        df = self.df.copy()
+        df = df[["b", "x"]]
+
+        # rename column 'b' (which will be used for the rows in the grid)
+        # by using a Unicode string:
+        unicode_row_label = u"\u01ff\u02ff\u03ff"
+        df = df.rename(columns={"b": unicode_row_label})
+
+        # ensure that the data frame columns have the expected names:
+        nt.assert_equal(list(df.columns), [unicode_row_label, "x"])
+
+        # plot the grid -- if successful, no UnicodeEncodingError should
+        # occur:
+        g = ag.FacetGrid(df, row=unicode_row_label)
+        g = g.map(plt.plot, "x")
+
+    def test_unicode_content_with_row_and_column(self):
+
+        df = self.df.copy()
+
+        # replace content of column 'a' (which will form the columns in the
+        # grid) by Unicode characters:
+        unicode_column_val = np.repeat((u'\u01ff', u'\u02ff', u'\u03ff'), 20)
+        df["a"] = unicode_column_val
+
+        # make sure that the replacement worked as expected:
+        nt.assert_equal(
+            list(df["a"]),
+            [u'\u01ff'] * 20 + [u'\u02ff'] * 20 + [u'\u03ff'] * 20)
+
+        # plot the grid -- if successful, no UnicodeEncodingError should
+        # occur:
+        g = ag.FacetGrid(df, col="a", row="b")
+        g = g.map(plt.plot, "x")
+
+    def test_unicode_content_no_rows(self):
+
+        df = self.df.copy()
+
+        # replace content of column 'a' (which will form the columns in the
+        # grid) by Unicode characters:
+        unicode_column_val = np.repeat((u'\u01ff', u'\u02ff', u'\u03ff'), 20)
+        df["a"] = unicode_column_val
+
+        # make sure that the replacement worked as expected:
+        nt.assert_equal(
+            list(df["a"]),
+            [u'\u01ff'] * 20 + [u'\u02ff'] * 20 + [u'\u03ff'] * 20)
+
+        # plot the grid -- if successful, no UnicodeEncodingError should
+        # occur:
+        g = ag.FacetGrid(df, col="a")
+        g = g.map(plt.plot, "x")
+
+    def test_unicode_content_no_columns(self):
+
+        df = self.df.copy()
+
+        # replace content of column 'a' (which will form the rows in the
+        # grid) by Unicode characters:
+        unicode_column_val = np.repeat((u'\u01ff', u'\u02ff', u'\u03ff'), 20)
+        df["b"] = unicode_column_val
+
+        # make sure that the replacement worked as expected:
+        nt.assert_equal(
+            list(df["b"]),
+            [u'\u01ff'] * 20 + [u'\u02ff'] * 20 + [u'\u03ff'] * 20)
+
+        # plot the grid -- if successful, no UnicodeEncodingError should
+        # occur:
+        g = ag.FacetGrid(df, row="b")
+        g = g.map(plt.plot, "x")
+
+    @skipif(not pandas_has_categoricals)
+    def test_categorical_column_missing_categories(self):
+
+        df = self.df.copy()
+        df['a'] = df['a'].astype('category')
+
+        g = ag.FacetGrid(df[df['a'] == 'a'], col="a", col_wrap=1)
+
+        nt.assert_equal(g.axes.shape, (len(df['a'].cat.categories),))
+
 
 class TestPairGrid(PlotTestCase):
 
@@ -780,6 +927,33 @@ class TestPairGrid(PlotTestCase):
         for ax in g3.diag_axes:
             nt.assert_equal(len(ax.patches), 40)
 
+        g4 = ag.PairGrid(self.df, hue="a")
+        g4.map_diag(plt.hist, histtype='step')
+
+        for ax in g4.diag_axes:
+            for ptch in ax.patches:
+                nt.assert_equal(ptch.fill, False)
+
+    @skipif(old_matplotlib)
+    def test_map_diag_color(self):
+
+        color = "red"
+        rgb_color = mpl.colors.colorConverter.to_rgba(color)
+
+        g1 = ag.PairGrid(self.df)
+        g1.map_diag(plt.hist, color=color)
+
+        for ax in g1.diag_axes:
+            for patch in ax.patches:
+                nt.assert_equals(patch.get_facecolor(), rgb_color)
+
+        g2 = ag.PairGrid(self.df)
+        g2.map_diag(kdeplot, color='red')
+
+        for ax in g2.diag_axes:
+            for line in ax.lines:
+                nt.assert_equals(line.get_color(), color)
+
     @skipif(old_matplotlib)
     def test_map_diag_and_offdiag(self):
 
@@ -972,7 +1146,7 @@ class TestPairGrid(PlotTestCase):
     def test_pairplot(self):
 
         vars = ["x", "y", "z"]
-        g = pairplot(self.df)
+        g = ag.pairplot(self.df)
 
         for ax in g.diag_axes:
             nt.assert_equal(len(ax.patches), 10)
@@ -1001,7 +1175,7 @@ class TestPairGrid(PlotTestCase):
     def test_pairplot_reg(self):
 
         vars = ["x", "y", "z"]
-        g = pairplot(self.df, kind="reg")
+        g = ag.pairplot(self.df, kind="reg")
 
         for ax in g.diag_axes:
             nt.assert_equal(len(ax.patches), 10)
@@ -1036,7 +1210,7 @@ class TestPairGrid(PlotTestCase):
     def test_pairplot_kde(self):
 
         vars = ["x", "y", "z"]
-        g = pairplot(self.df, diag_kind="kde")
+        g = ag.pairplot(self.df, diag_kind="kde")
 
         for ax in g.diag_axes:
             nt.assert_equal(len(ax.lines), 1)
@@ -1066,12 +1240,12 @@ class TestPairGrid(PlotTestCase):
 
         vars = ["x", "y", "z"]
         markers = ["o", "x", "s", "d"]
-        g = pairplot(self.df, hue="a", vars=vars, markers=markers)
+        g = ag.pairplot(self.df, hue="a", vars=vars, markers=markers)
         nt.assert_equal(g.hue_kws["marker"], markers)
         plt.close("all")
 
         with nt.assert_raises(ValueError):
-            g = pairplot(self.df, hue="a", vars=vars, markers=markers[:-2])
+            g = ag.pairplot(self.df, hue="a", vars=vars, markers=markers[:-2])
 
 
 class TestJointGrid(PlotTestCase):
@@ -1102,6 +1276,11 @@ class TestJointGrid(PlotTestCase):
         npt.assert_array_equal(g.x, self.x)
         npt.assert_array_equal(g.y, self.y)
 
+    def test_margin_grid_from_dataframe_bad_variable(self):
+
+        with nt.assert_raises(ValueError):
+            g = ag.JointGrid("x", "bad_column", self.data)
+
     def test_margin_grid_axis_labels(self):
 
         g = ag.JointGrid("x", "y", self.data)
@@ -1208,3 +1387,108 @@ class TestJointGrid(PlotTestCase):
 
         nt.assert_equal(joint_bounds[2], marg_x_bounds[2])
         nt.assert_equal(joint_bounds[3], marg_y_bounds[3])
+
+
+class TestJointPlot(PlotTestCase):
+
+    rs = np.random.RandomState(sum(map(ord, "jointplot")))
+    x = rs.randn(100)
+    y = rs.randn(100)
+    data = pd.DataFrame(dict(x=x, y=y))
+
+    def test_scatter(self):
+
+        g = ag.jointplot("x", "y", self.data)
+        nt.assert_equal(len(g.ax_joint.collections), 1)
+
+        x, y = g.ax_joint.collections[0].get_offsets().T
+        npt.assert_array_equal(self.x, x)
+        npt.assert_array_equal(self.y, y)
+
+        x_bins = _freedman_diaconis_bins(self.x)
+        nt.assert_equal(len(g.ax_marg_x.patches), x_bins)
+
+        y_bins = _freedman_diaconis_bins(self.y)
+        nt.assert_equal(len(g.ax_marg_y.patches), y_bins)
+
+    def test_reg(self):
+
+        g = ag.jointplot("x", "y", self.data, kind="reg")
+        nt.assert_equal(len(g.ax_joint.collections), 2)
+
+        x, y = g.ax_joint.collections[0].get_offsets().T
+        npt.assert_array_equal(self.x, x)
+        npt.assert_array_equal(self.y, y)
+
+        x_bins = _freedman_diaconis_bins(self.x)
+        nt.assert_equal(len(g.ax_marg_x.patches), x_bins)
+
+        y_bins = _freedman_diaconis_bins(self.y)
+        nt.assert_equal(len(g.ax_marg_y.patches), y_bins)
+
+        nt.assert_equal(len(g.ax_joint.lines), 1)
+        nt.assert_equal(len(g.ax_marg_x.lines), 1)
+        nt.assert_equal(len(g.ax_marg_y.lines), 1)
+
+    def test_resid(self):
+
+        g = ag.jointplot("x", "y", self.data, kind="resid")
+        nt.assert_equal(len(g.ax_joint.collections), 1)
+        nt.assert_equal(len(g.ax_joint.lines), 1)
+        nt.assert_equal(len(g.ax_marg_x.lines), 0)
+        nt.assert_equal(len(g.ax_marg_y.lines), 1)
+
+    def test_hex(self):
+
+        g = ag.jointplot("x", "y", self.data, kind="hex")
+        nt.assert_equal(len(g.ax_joint.collections), 1)
+
+        x_bins = _freedman_diaconis_bins(self.x)
+        nt.assert_equal(len(g.ax_marg_x.patches), x_bins)
+
+        y_bins = _freedman_diaconis_bins(self.y)
+        nt.assert_equal(len(g.ax_marg_y.patches), y_bins)
+
+    def test_kde(self):
+
+        g = ag.jointplot("x", "y", self.data, kind="kde")
+
+        nt.assert_true(len(g.ax_joint.collections) > 0)
+        nt.assert_equal(len(g.ax_marg_x.collections), 1)
+        nt.assert_equal(len(g.ax_marg_y.collections), 1)
+
+        nt.assert_equal(len(g.ax_marg_x.lines), 1)
+        nt.assert_equal(len(g.ax_marg_y.lines), 1)
+
+    def test_color(self):
+
+        g = ag.jointplot("x", "y", self.data, color="purple")
+
+        purple = mpl.colors.colorConverter.to_rgb("purple")
+        scatter_color = g.ax_joint.collections[0].get_facecolor()[0, :3]
+        nt.assert_equal(tuple(scatter_color), purple)
+
+        hist_color = g.ax_marg_x.patches[0].get_facecolor()[:3]
+        nt.assert_equal(hist_color, purple)
+
+    def test_annotation(self):
+
+        g = ag.jointplot("x", "y", self.data)
+        nt.assert_equal(len(g.ax_joint.legend_.get_texts()), 1)
+
+        g = ag.jointplot("x", "y", self.data, stat_func=None)
+        nt.assert_is(g.ax_joint.legend_, None)
+
+    def test_hex_customise(self):
+
+        # test that default gridsize can be overridden
+        g = ag.jointplot("x", "y", self.data, kind="hex",
+                         joint_kws=dict(gridsize=5))
+        nt.assert_equal(len(g.ax_joint.collections), 1)
+        a = g.ax_joint.collections[0].get_array()
+        nt.assert_equal(28, a.shape[0])  # 28 hexagons expected for gridsize 5
+
+    def test_bad_kind(self):
+
+        with nt.assert_raises(ValueError):
+            ag.jointplot("x", "y", self.data, kind="not_a_kind")
