@@ -5,25 +5,29 @@ import shutil
 
 import numpy as np
 import pandas as pd
+import matplotlib as mpl
 import matplotlib.pyplot as plt
 import nose
 import nose.tools as nt
 from nose.tools import assert_equal, raises
 import numpy.testing as npt
-import pandas.util.testing as pdt
+try:
+    import pandas.testing as pdt
+except ImportError:
+    import pandas.util.testing as pdt
 
 from distutils.version import LooseVersion
 pandas_has_categoricals = LooseVersion(pd.__version__) >= "0.15"
 
-from pandas.util.testing import network
-from ..utils import get_dataset_names, load_dataset
 
 try:
     from bs4 import BeautifulSoup
 except ImportError:
     BeautifulSoup = None
 
+from . import PlotTestCase
 from .. import utils, rcmod
+from ..utils import get_dataset_names, load_dataset, _network
 
 
 a_norm = np.random.randn(100)
@@ -109,7 +113,15 @@ def test_iqr():
     assert_equal(iqr, 2)
 
 
-class TestSpineUtils(object):
+def test_str_to_utf8():
+    """Test the to_utf8 function: string to Unicode"""
+    s = "\u01ff\u02ff"
+    u = utils.to_utf8(s)
+    assert_equal(type(s), type(str()))
+    assert_equal(type(u), type(u"\u01ff\u02ff"))
+
+
+class TestSpineUtils(PlotTestCase):
 
     sides = ["left", "right", "bottom", "top"]
     outer_sides = ["top", "right"]
@@ -134,8 +146,6 @@ class TestSpineUtils(object):
         for side in self.sides:
             nt.assert_true(~ax.spines[side].get_visible())
 
-        plt.close("all")
-
     def test_despine_specific_axes(self):
         f, (ax1, ax2) = plt.subplots(2, 1)
 
@@ -149,8 +159,6 @@ class TestSpineUtils(object):
         for side in self.inner_sides:
             nt.assert_true(ax2.spines[side].get_visible())
 
-        plt.close("all")
-
     def test_despine_with_offset(self):
         f, ax = plt.subplots()
 
@@ -168,7 +176,18 @@ class TestSpineUtils(object):
             else:
                 nt.assert_equal(new_position, self.original_position)
 
-        plt.close("all")
+    def test_despine_side_specific_offset(self):
+
+        f, ax = plt.subplots()
+        utils.despine(ax=ax, offset=dict(left=self.offset))
+
+        for side in self.sides:
+            is_visible = ax.spines[side].get_visible()
+            new_position = ax.spines[side].get_position()
+            if is_visible and side == "left":
+                nt.assert_equal(new_position, self.offset_position)
+            else:
+                nt.assert_equal(new_position, self.original_position)
 
     def test_despine_with_offset_specific_axes(self):
         f, (ax1, ax2) = plt.subplots(2, 1)
@@ -184,7 +203,6 @@ class TestSpineUtils(object):
             else:
                 nt.assert_equal(ax2.spines[side].get_position(),
                                 self.original_position)
-        plt.close("all")
 
     def test_despine_trim_spines(self):
         f, ax = plt.subplots()
@@ -196,8 +214,6 @@ class TestSpineUtils(object):
             bounds = ax.spines[side].get_bounds()
             nt.assert_equal(bounds, (1, 3))
 
-        plt.close("all")
-
     def test_despine_trim_inverted(self):
 
         f, ax = plt.subplots()
@@ -210,8 +226,6 @@ class TestSpineUtils(object):
             bounds = ax.spines[side].get_bounds()
             nt.assert_equal(bounds, (1, 3))
 
-        plt.close("all")
-
     def test_despine_trim_noticks(self):
 
         f, ax = plt.subplots()
@@ -220,48 +234,6 @@ class TestSpineUtils(object):
         utils.despine(trim=True)
         nt.assert_equal(ax.get_yticks().size, 0)
 
-    def test_offset_spines_warns(self):
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always", category=UserWarning)
-
-            f, ax = plt.subplots()
-            utils.offset_spines(offset=self.offset)
-            nt.assert_true('deprecated' in str(w[0].message))
-            nt.assert_true(issubclass(w[0].category, UserWarning))
-
-        plt.close('all')
-
-    def test_offset_spines(self):
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always", category=UserWarning)
-            f, ax = plt.subplots()
-
-            for side in self.sides:
-                nt.assert_equal(ax.spines[side].get_position(),
-                                self.original_position)
-
-            utils.offset_spines(offset=self.offset)
-
-            for side in self.sides:
-                nt.assert_equal(ax.spines[side].get_position(),
-                                self.offset_position)
-
-        plt.close("all")
-
-    def test_offset_spines_specific_axes(self):
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always", category=UserWarning)
-            f, (ax1, ax2) = plt.subplots(2, 1)
-
-            utils.offset_spines(offset=self.offset, ax=ax2)
-
-            for side in self.sides:
-                nt.assert_equal(ax1.spines[side].get_position(),
-                                self.original_position)
-                nt.assert_equal(ax2.spines[side].get_position(),
-                                self.offset_position)
-        plt.close("all")
-
 
 def test_ticklabels_overlap():
 
@@ -286,6 +258,7 @@ def test_ticklabels_overlap():
 def test_categorical_order():
 
     x = ["a", "c", "c", "b", "a", "d"]
+    y = [3, 2, 5, 1, 4]
     order = ["a", "b", "c", "d"]
 
     out = utils.categorical_order(x)
@@ -303,6 +276,15 @@ def test_categorical_order():
     out = utils.categorical_order(pd.Series(x))
     nt.assert_equal(out, ["a", "c", "b", "d"])
 
+    out = utils.categorical_order(y)
+    nt.assert_equal(out, [1, 2, 3, 4, 5])
+
+    out = utils.categorical_order(np.array(y))
+    nt.assert_equal(out, [1, 2, 3, 4, 5])
+
+    out = utils.categorical_order(pd.Series(y))
+    nt.assert_equal(out, [1, 2, 3, 4, 5])
+
     if pandas_has_categoricals:
         x = pd.Categorical(x, order)
         out = utils.categorical_order(x)
@@ -342,7 +324,7 @@ if LooseVersion(pd.__version__) >= "0.15":
         finally:
             shutil.rmtree(tmpdir)
 
-    @network(url="https://github.com/mwaskom/seaborn-data")
+    @_network(url="https://github.com/mwaskom/seaborn-data")
     def test_get_dataset_names():
         if not BeautifulSoup:
             raise nose.SkipTest("No BeautifulSoup available for parsing html")
@@ -350,7 +332,7 @@ if LooseVersion(pd.__version__) >= "0.15":
         assert(len(names) > 0)
         assert(u"titanic" in names)
 
-    @network(url="https://github.com/mwaskom/seaborn-data")
+    @_network(url="https://github.com/mwaskom/seaborn-data")
     def test_load_datasets():
         if not BeautifulSoup:
             raise nose.SkipTest("No BeautifulSoup available for parsing html")
@@ -362,7 +344,7 @@ if LooseVersion(pd.__version__) >= "0.15":
             # yield check_load_dataset, name
             check_load_dataset(name)
 
-    @network(url="https://github.com/mwaskom/seaborn-data")
+    @_network(url="https://github.com/mwaskom/seaborn-data")
     def test_load_cached_datasets():
         if not BeautifulSoup:
             raise nose.SkipTest("No BeautifulSoup available for parsing html")
@@ -373,3 +355,22 @@ if LooseVersion(pd.__version__) >= "0.15":
             # does not get in effect, so we need to call explicitly
             # yield check_load_dataset, name
             check_load_cached_dataset(name)
+
+
+def test_relative_luminance():
+    """Test relative luminance."""
+    out1 = utils.relative_luminance("white")
+    assert_equal(out1, 1)
+
+    out2 = utils.relative_luminance("#000000")
+    assert_equal(out2, 0)
+
+    out3 = utils.relative_luminance((.25, .5, .75))
+    nose.tools.assert_almost_equal(out3, 0.201624536)
+
+    rgbs = mpl.cm.RdBu(np.linspace(0, 1, 10))
+    lums1 = [utils.relative_luminance(rgb) for rgb in rgbs]
+    lums2 = utils.relative_luminance(rgbs)
+
+    for lum1, lum2 in zip(lums1, lums2):
+        nose.tools.assert_almost_equal(lum1, lum2)
